local prefix = "DS" --Prefix for sending addon msgs
local prepend = "[DS]: " --Prefix for sending msgs in chat channels (player visible)
local currentBG = {} --List of the current players in your BG key=Name-Realm value=Faction(0 Horde, 1 Alliance)
local currentInBG = 0 --Track the current number of people in your BG (or arena)
local currentFaction = 0 --Horde by default. Lok'Tar!
local MAX_DISPLAY_PLAYERS = 17 --Max number of people to display at one time
local wonYet = false --If the current BG has been won (to prevent redundancy of calculating wins twice)
local showAllies = true
local showOpponents = true

DS = LibStub("AceAddon-3.0"):NewAddon("DS", "AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0", "AceHook-3.0")

--Creates all the 'buttons' for the List in the main window.
do
	local entry = CreateFrame("Button", "$parentEntry1", DangerScout_MainTableList, "DS_TableEntry")
	entry:SetID(1)
	entry:SetPoint("TOPLEFT", 4, -28)
	for i = 2, MAX_DISPLAY_PLAYERS do
		local entry = CreateFrame("Button", "$parentEntry"..i, DangerScout_MainTableList, "DS_TableEntry")
		entry:SetID(i)
		entry:SetPoint("TOP", "$parentEntry"..(i-1), "BOTTOM")
	end
end

function DS:OnInitialize()
	DS:RegisterChatCommand("ds", "SlashHandler")
	self.db = LibStub("AceDB-3.0"):New("DSDB")
	
	--Make sure all the tables exist
	if(self.db.factionrealm.opponents == nil) then
		self.db.factionrealm.opponents = {}
	end
	if(self.db.factionrealm.allies == nil) then
		self.db.factionrealm.allies = {}
	end
	
	--Add the player to their allies table.
	local realm = GetRealmName('player')
	if(self.db.factionrealm.allies[realm] == nil) then
		self.db.factionrealm.allies[realm] = {}
		local name = UnitName('player')
		local fac = UnitFactionGroup('player')
		local class, eclass = UnitClass('player')
		if fac == "Horde" then
			currentFaction = 0
		else
			currentFaction = 1
		end
		self.db.factionrealm.allies[realm][name] = {0, 0, currentFaction, eclass, 0, 0}
	end
end

function DS:OnEnable()
	--Register all the events listeners
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")
end

function DS:PLAYER_ENTERING_WORLD()
	--TODO: Put this somewhere it'll only be read once
	local englishFac, localizedFac = UnitFactionGroup('player')
	if englishFac == "Alliance" then
		currentFaction = 1
	end
	DS:Print("Fight for the " .. localizedFac .. "! DangerScout has loaded!")
end

function DS:UPDATE_BATTLEFIELD_STATUS()
	local winner = GetBattlefieldWinner()
	--If nil, do nothing, nobody has won yet, if 0, horde, if 1, alliance.
	--Add 1 to all the wins for the people in the currentBG on the respective faction.
	if(wonYet == false) then
		if(winner ~= nil) then
			DS:Scan()
			wonYet = true
			for key, value in pairs(currentBG) do
				local name, realm = SplitName(key)
				if(winner == currentFaction) then
					self.db.factionrealm.allies[realm][name][2] = self.db.factionrealm.allies[realm][name][2] + 1
				else
					self.db.factionrealm.opponents[realm][name][2] = self.db.factionrealm.opponents[realm][name][2] + 1
				end
			end
		end
	end
end

function DS:CHAT_MSG_ADDON(event, pre, msg, channel, sender)
	if pre == prefix then
		DS:Print(string.format("In [%s} [%s] sent: %s - %s", channel, sender, pre, msg))
	end
end

function DS:ZONE_CHANGED_NEW_AREA(event)
	local zoneName = GetZoneText();
	for _, area in pairs(ARENAS) do
		if(zoneName == area) then
			--Arena has not been tested yet
			getglobal("DangerScout_Main"):Show()
			--self.timer = self:ScheduleTimer("Scan", 60)
			wonYet = false
			--self:Scan()
			--If we are in an arena set a timer to only scan once (as matches are short)
			return
		end
	end
	for _, area in pairs(BATTLE_GROUNDS) do
		if(zoneName == area) then
			getglobal("DangerScout_Main"):Show()
			self.timer = self:ScheduleRepeatingTimer("Scan", 60)
			wonYet = false
			self:Scan()
			--If we are in a bg, scan every 60s as people leave and enter
			return
		end
	end
	if(timer ~= nil) then
		self:CancelTimer(self.timer)
		--If not in either appropriate zone, remove the timer.
		--TODO: Test, this feature may not be working as intended
	end
end

function DS:Scan()
	currentInBG = GetNumBattlefieldScores() --Get the number of players in the current BG
	local alliesIndex = 0
	local opponentIndex = 0
	
	if(currentBG[0] == nil) then
		currentBG[0] = {}
	end
	if(currentBG[1] == nil) then
		currentBG[1] = {}
	end
	for i = 1, currentInBG do
	--TODO: Get if player is in arena, if so, add 'opponents' to allies list?
		local name, killingBlows, honorableKills, deaths, honorGained, faction, rank, race, class = GetBattlefieldScore(i)
		local player, realm = SplitName(name)
		--if(currentBG[faction] == nil) then --If they haven't already been scanned in the current BG
			if faction == currentFaction then
				if(self.db.factionrealm.allies[realm] == nil) then --If the realm doesnt exist, create it
					self.db.factionrealm.allies[realm] = {}
				end
				if self.db.factionrealm.allies[realm][player] == nil then --If the player doesnt exist, create them and start them at {seen, wins, faction, class, KBs, deaths}
					self.db.factionrealm.allies[realm][player] = {1, 0, faction, class, 0, 0}
				else
					self.db.factionrealm.allies[realm][player][1] = self.db.factionrealm.allies[realm][player][1] + 1 --We've already seen them, add 1 to their times seen
				end
				currentBG[faction][alliesIndex] = name; --Add them to the list
				alliesIndex = alliesIndex + 1
			else --Do the same, but add them to the opponents table if they are on the opposite faction
				if(self.db.factionrealm.opponents[realm] == nil) then
					self.db.factionrealm.opponents[realm] = {}
				end
				if self.db.factionrealm.opponents[realm][player] == nil then
					self.db.factionrealm.opponents[realm][player] = {1, 0, faction, class, 0, 0}
				else
					self.db.factionrealm.opponents[realm][player][1] = self.db.factionrealm.opponents[realm][player][1] + 1
				end
				currentBG[faction][opponentIndex] = name;
				opponentIndex = opponentIndex + 1
			end
		--end
	end
	--[[
	--Clean out any leavers
	for key, _ in pairs(currentBG) do
		local seen = false
		for i = 1, currentInBG do
			local name = GetBattlefieldScore(i)
			if(name == key) then
				seen = true
				break
			end
		end
		if seen == false then
			table.remove(currentBG, key)
		end
	end
	--]]
	self:Update() --Update our UI
end

function DS:PrintCurrent()
 for name, value in pairs(currentBG) do
	DS:Print(string.format("%s %s", name, value))
 end
end

function Update()
	DS:Update() --Hackiness to use in XML
end

function DS:Update()
	local displayNum
	if(showAllies) then
		displayNum = currentInBG / 2;
	end
	if(showOpponents) then
		displayNum = displayNum + (currentInBG / 2)
	end
	FauxScrollFrame_Update(DangerScout_MainTableListScrollFrame, currentInBG, MAX_DISPLAY_PLAYERS, 
	24, "DangerScout_MainTableListEntry", 684, 700, 
	DangerScout_MainTableListHeaderLosses, 218, 234
	)
	
	local index = 1
	for i = 1, MAX_DISPLAY_PLAYERS do --Only loop through the number of players there are slots on our UI
		local player, faction;
		
		if(showAllies) then
			player = currentBG[currentFaction][i + DangerScout_MainTableListScrollFrame.offset];
			faction = currentFaction;
		end
		if(showOpponents) then
			player = currentBG[GetOtherFaction()][i + DangerScout_MainTableListScrollFrame.offset];
			faction = GetOtherFaction();
		end
		
		local frame = getglobal("DangerScout_MainTableListEntry"..i) --Get the button we will use eg. DangerScout_MainTableListEntry(1-17)
		if player then
			frame:Show() --Show the frame, is hidden on init (when it has no data)
			local name, realm = SplitName(player)

			getglobal(frame:GetName().."PlayerName"):SetText(name)
			local wins, losses = -1 --Place holder so we know if data isnt being taken from the DB
			if(faction == currentFaction) then
				wins = self.db.factionrealm.allies[realm][name][2]
				losses = self.db.factionrealm.allies[realm][name][1]-self.db.factionrealm.allies[realm][name][2] --Subtract wins from seen to get losses
			else
				wins = self.db.factionrealm.opponents[realm][name][2]
				losses = self.db.factionrealm.opponents[realm][name][1]-self.db.factionrealm.opponents[realm][name][2] 
			end
			if faction == 0 then --Set the players name the color of their faction
				getglobal(frame:GetName().."PlayerName"):SetTextColor(1, 0, 0, 1)
			else
				getglobal(frame:GetName().."PlayerName"):SetTextColor(0.27, 0.53, 0.94, 1)
			end
			if(name == UnitName('player')) then
				getglobal(frame:GetName().."PlayerName"):SetTextColor(0.27, 0.81, 0.31, 1)
			end
			
			getglobal(frame:GetName().."NumWins"):SetText(wins)
			getglobal(frame:GetName().."NumLosses"):SetText(losses)
			--if entry.isSelected then
				--getglobal(frame:GetName().."BG"):Show()
			--else
				--getglobal(frame:GetName().."BG"):Hide()
			--end
		else
			frame:Hide() --If no frame exists hide it? Derp.
		end
	end
end

function DS:SlashHandler(input)
	local args = {}
	i = 0;
	for token in string.gmatch(input, "[^%s]+") do
		args[i] = token
		i = i + 1
	end

	if args[0] == 'help' then
		DS:Print("Valid commands:")
		DS:Print("show - to show the main window.")
	elseif args[0] == 'show' then
		DangerScout_Main:Show()
	else
		DS:Print("Invalid command. Try '/ds help' for info.")
	end
end

function DS:PurgeDB()
	--Remove all data from the DB. No srsly guys. ALL OF IT. Be careful what you do...
	self.db.factionrealm.opponents = {}
	self.db.factionrealm.allies = {}
	DS:Scan()
end

function SplitName(name)
	--Split a 'Ecoshock-Mal'Ganis' string into 'Ecoshock' 'Mal'Ganis'.. if no realm is in name, 
		--that means you are on the same realm as the host player therefore just return the current realm
	local charName, realmName = strsplit('-', name, 2)
	if(realmName == nil) then
		realmName = GetRealmName()
	end
	return charName, realmName
end

function GetOtherFaction()
	if(currentFaction == 1) then
		return 0
	else
		return 1;
	end
end

function AddToFaction(faction)
	return faction + 1
end

function DS:ShowDBPurge()
	StaticPopup_Show("DS_CLEAR_DB")
end

function DS:ClickAllies()
	showAllies = not showAllies
end

function DS:ClickOpponents()
	showOpponents = not showOpponents
end

StaticPopupDialogs["DS_CLEAR_DB"] = {
	text = DS_CLEAR_DB_DIALOG,
	button1 = ACCEPT,
	button2 = CANCEL,
	OnAccept = function(self)
		DS:PurgeDB()
	end,
}